!!-----------------------------------------------------------------------
!!     Copyright (C) 2003-2014, ENPC - INRIA - EDF R&D
!!     Author(s): Shupeng Zhu
!!
!!     This file is part of the Size Composition Resolved Aerosol Model (SCRAM), a
!!     component of the air quality modeling system Polyphemus.
!!
!!     Polyphemus is developed in the INRIA - ENPC joint project-team
!!     CLIME and in the ENPC - EDF R&D joint laboratory CEREA.
!!
!!     Polyphemus is free software; you can redistribute it and/or modify
!!     it under the terms of the GNU General Public License as published
!!     by the Free Software Foundation; either version 2 of the License,
!!     or (at your option) any later version.
!!
!!     Polyphemus is distributed in the hope that it will be useful, but
!!     WITHOUT ANY WARRANTY; without even the implied warranty of
!!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
!!     General Public License for more details.
!!
!!     For more information, visit the Polyphemus web site:
!!     http://cerea.enpc.fr/polyphemus/
!!-----------------------------------------------------------------------
!!
!!     -- DESCRIPTION
!!    This module contains methods to solve particle condensation/evaporation
!!    based on the gas/aerosol bulk equilibrium method.
!!-----------------------------------------------------------------------
Module Bulkequibrium
  use Discretization
  use Initialization
  use Thermodynamics  
  implicit none
contains

  subroutine bulkequi_org()
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!     This subroutine solves gas/aerosol bulk equilibrium for organic species
!     based on H2O method.
!     equilibrium will be established between all size bin and bulk gas
!     mass flux will be redistributed into each cell based on rates
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!------------------------------------------------------------------------   
    implicit none
    
    integer::j,jesp,iter,s
    double precision::dq(N_aerosol),qext(N_aerosol),qextold(N_aerosol)
    double precision::qaero(N_aerosol),qgas(N_aerosol)
    double precision::ce_kernal_coef_tot(N_aerosol)
    integer with_oligomerization,thermodynamic_model
    parameter (with_oligomerization=0) !with oligomerization if with_oligomerization=1
    parameter (thermodynamic_model=1) !thermodynamic_model = 0 ideal, thermodynamic_model=1 UNIFAC
    double precision organion, watorg, proton, lwc
    integer, dimension(:), allocatable :: eq_species! species compute with equilibrium
    integer ::nesp_eq!number of species at equilibirum
  
!!     ******zero init
    do jesp=1,N_aerosol
	dq(jesp)=0.D0
	qgas(jesp)=0.d0
	qext(jesp)=0.D0
	qextold(jesp)=0.D0
	qaero(jesp)=0.d0
	ce_kernal_coef_tot(jesp)=0.D0!TINYA
    end do

  !     ****** allocate equilibrium species list      
    nesp_eq=nesp_pom+nesp_pankow+nesp_aec
    allocate(eq_species(nesp_eq))

    do s=1,nesp_aec
      eq_species(s)=aec_species(s)
    enddo
    do s=1,nesp_pankow
      eq_species(s+nesp_aec)=pankow_species(s)
    enddo
    do s=1,nesp_pom
      eq_species(s+nesp_aec+nesp_pankow)=poa_species(s)
    enddo

  !compute local equi

  !compute only c/e coefficients
    do j = 1,N_size	!FOR OGANIC
      do jesp=E1, E2
	call COMPUTE_CONDENSATION_TRANSFER_RATE(&
	      diffusion_coef(jesp), &! diffusion coef (m2.s-1)
	      quadratic_speed(jesp),& ! quadratic mean speed (m.s-1)
	      accomodation_coefficient(jesp),& ! accomadation coef (adim)
	      wet_diameter(j),   & ! wet aero diameter (µm)
	      ce_kernal_coef(j,jesp) ) ! c/e kernel coef (m3.s-1)
	ce_kernal_coef_tot(jesp)= ce_kernal_coef_tot(jesp)&! compute total ce_kernal_coef coef
		    +ce_kernal_coef(j,jesp)*concentration_number(j)
      enddo
    enddo

    do jesp=E1,E2!organic
      do j=1,N_size
	qaero(jesp)=qaero(jesp)+concentration_mass(j,jesp)
      enddo
      qgas(jesp)=concentration_gas(jesp)!initial gas for H2O
      qextold(jesp)=qaero(jesp)
    end do
    if(sulfate_computation.eq.0) then
	qgas(ESO4)=concentration_gas(ESO4)
    else
	qgas(ESO4)=0.d0
    endif
    qgas(EH2O)=0.0
    organion = 0.D0
    watorg = 0.D0
    proton = 0.D0

    do iter = 1,NITER_AEC_AQ
      call isoropia_drv(N_aerosol,&
	    qaero,qgas,organion, watorg, proton, lwc, Relative_Humidity, Temperature)
      call aec_drv(N_aerosol,&
	  0, qaero, qgas, proton, lwc, organion,watorg, Relative_Humidity, Temperature,&
	  with_oligomerization, thermodynamic_model)
    enddo
    qaero(N_aerosol)=lwc
    do iter = 1,NITER_AEC_DRY
      call poa_drv(N_aerosol,qaero, qgas,soa_part_coef,Temperature,vaporization_enthalpy)
      call pankow_drv(N_aerosol,qaero, qgas,soa_part_coef)
      call aec_drv(N_aerosol,&
	1, qaero, qgas, proton, lwc, organion,watorg, Relative_Humidity, Temperature,&
	with_oligomerization, thermodynamic_model)
    enddo
    qgas(EH2O)=0.0

      !for organic
    do s=1,nesp_eq
      jesp=eq_species(s)
      if (IsNaN(qaero(jesp))) qaero(jesp)=0.d0
      qext(jesp)=qaero(jesp)
      concentration_gas(jesp)=qgas(jesp)!new qgas is used in N_aerosol bin
      if(qext(jesp).gt.0.d0) then
	dq(jesp)=qext(jesp)-qextold(jesp)! compute delta aero conc
      else
	dq(jesp)=-qextold(jesp)
      endif
      m_grow_cond=m_grow_cond+dq(jesp)
    enddo

!     ******redistribute on each cell according to Rates
    call bulkequi_redistribution(concentration_number,concentration_mass,&
    nesp_eq,eq_species,N_size,dq,ce_kernal_coef,ce_kernal_coef_tot)

!update wet diameter at the end of current c/e cycle
    call compute_wet_mass_diameter(1,N_size,concentration_mass,concentration_number,&
	concentration_inti,wet_mass,wet_diameter,wet_volume,cell_diam_av)
	  
  end subroutine bulkequi_org

  subroutine bulkequi_inorg()
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!     This subroutine solves gas/aerosol bulk equilibrium for inorganic species
!     based on ISORROPIA method.
!     equilibrium will be established between all size bin < ICUT and bulk gas
!     mass flux will be redistributed into each cell based on rates
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!------------------------------------------------------------------------   
    implicit none
    integer::j,jesp,s
    double precision::dq(N_aerosol),qext(N_aerosol),qextold(N_aerosol)
    double precision::qaero(N_aerosol),qgas(N_aerosol)
    double precision::ce_kernal_coef_tot(N_aerosol)
    double precision::aatoteq,qgasa,qgasi
    integer with_oligomerization,thermodynamic_model
    parameter (with_oligomerization=0) !with oligomerization if with_oligomerization=1
    parameter (thermodynamic_model=1) !thermodynamic_model = 0 ideal, thermodynamic_model=1 UNIFAC
    double precision organion, watorg, proton, lwc
    integer, dimension(:), allocatable :: eq_species! species compute with equilibrium
    integer ::nesp_eq!number of species at equilibirum
  
!!     ******zero init
    do jesp=1,N_aerosol
      dq(jesp)=0.D0
      qgas(jesp)=0.d0
      qext(jesp)=0.D0
      qextold(jesp)=0.D0
      qaero(jesp)=0.d0
      ce_kernal_coef_tot(jesp)=0.D0
    end do

  !     ****** if sulfate computed dynamically avoid it
  !     ****** allocate equilibrium species list
    if (sulfate_computation.eq.1) then
      nesp_eq=nesp_isorropia-2
    else
      nesp_eq=nesp_isorropia-1
    endif

    allocate(eq_species(nesp_eq))
    do jesp=1,nesp_eq
      eq_species(jesp)=isorropia_species(nesp_isorropia-nesp_eq+jesp)
    enddo

  !compute only c/e coefficients

    ce_kernal_coef_tot(jesp)=0.d0

    do s=1, nesp_isorropia
      jesp=isorropia_species(s)
      do j = 1,ICUT	!FOR INOGANIC
	call COMPUTE_CONDENSATION_TRANSFER_RATE(&
	      diffusion_coef(jesp), &! diffusion coef (m2.s-1)
	      quadratic_speed(jesp),& ! quadratic mean speed (m.s-1)
	      accomodation_coefficient(jesp),& ! accomadation coef (adim)
	      wet_diameter(j),   & ! wet aero diameter (µm)
	      ce_kernal_coef(j,jesp) ) ! c/e kernel coef (m3.s-1)
	if(jesp.ne.ESO4) then
	  ! compute total ce_kernal_coef coef (s-1) EXCEPT SO4
	  ce_kernal_coef_tot(jesp)= ce_kernal_coef_tot(jesp)&
		    +ce_kernal_coef(j,jesp)*concentration_number(j)
	endif
      enddo
    enddo

  !     ****** if sulfate computed dynamically avoid it
    if (sulfate_computation.eq.0) then
      do j=1,N_size
	jesp=ESO4!
	ce_kernal_coef_tot(jesp)= ce_kernal_coef_tot(jesp)&
	    +ce_kernal_coef(j,jesp)*concentration_number(j)
      end do
      qgasa=0.d0
      qgasi=concentration_gas(ESO4)
    else
      qgasi=0.d0
    endif

    !compute total mass of each species
    do s=1,nesp_isorropia!inorganic
      jesp=isorropia_species(s)
      do j=1,ICUT
	qaero(jesp)=qaero(jesp)+concentration_mass(j,jesp)
      enddo
      qgas(jesp)=concentration_gas(jesp)
      qextold(jesp)=qaero(jesp)
    end do

    if(sulfate_computation.eq.0) then
      !compute apparent gas concentration of sulfate
      do j=1,ICUT
	aatoteq=aatoteq+ce_kernal_coef(j,ESO4)*concentration_number(j)
      enddo
      qgas(ESO4)=concentration_gas(ESO4)*aatoteq/ce_kernal_coef_tot(ESO4)
      qgasa=qgas(ESO4)
    else
      qgas(ESO4)=0.d0
    endif
    organion= 0.D0
    watorg = 0.D0
    proton= 0.D0

  !eqilibirum for inorganic
    call isoropia_drv(N_aerosol,&
	  qaero,qgas,organion, watorg, proton, lwc, Relative_Humidity, Temperature)
    qaero(N_aerosol)=lwc
    qgas(EH2O)=0.0

  !     ******redistribute on each cell according to Rates

    if(sulfate_computation.eq.0) then
      ce_kernal_coef_tot(ESO4)=aatoteq!for later redistribution
    endif
    do s=1, nesp_eq
      jesp=eq_species(s)
      if (IsNaN(qaero(jesp))) qaero(jesp)=0.d0!detect NaN case
      qext(jesp)=qaero(jesp)
      concentration_gas(jesp)=qgas(jesp)!new qgas is used in N_aerosol bin
      if(qext(jesp).gt.0.d0) then
	dq(jesp)=qext(jesp)-qextold(jesp)! compute delta aero conc
      else
	dq(jesp)=-qextold(jesp)
      endif
      m_grow_cond=m_grow_cond+dq(jesp)
    enddo
    call bulkequi_redistribution(concentration_number,concentration_mass,&
    nesp_eq,eq_species,ICUT,dq,ce_kernal_coef,ce_kernal_coef_tot)
  ! give back initial SO4 gas conc
  ! minus that consumed by equi bins
    if(sulfate_computation.eq.1) then
    concentration_gas(ESO4)=qgasi-qgasa
    endif
  end subroutine bulkequi_inorg
  
  subroutine bulkequi_redistribution(c_number,c_mass,nesp_eq,eq_species,end_bin,dq,AAi,ce_kernal_coef_tot)
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!     This subroutine redistribute bulk mass variations into each bins
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!
!     c_number: aerosol number concentration(#/m^3)
!     c_mass: aerosol mass concentration(g/m^3)
!     nesp_eq: number of species at equilibirum
!     eq_species: the list of species pointer at equilibirum
!     end_bin: marks the number of bins concerned during the redistribution
!     dq: bulk mass variations
!     AAi: c/e kernel coefficient          ([m3.s-1]).
!     ce_kernal_coef_tot: sum of c/e kernel coefficient          ([m3.s-1]).
!------------------------------------------------------------------------   
    implicit none
    integer::j,s,jesp,end_bin,iclip
    integer::nesp_eq
    integer::eq_species(nesp_eq)
    double precision::totaer,temp_mass
    double precision::ce_kernal_coef_tot(N_aerosol)
    double precision::dq(N_aerosol)
    double precision::AAi(N_size,N_aerosol)
    double precision::frac(N_size,N_aerosol)
    double precision::c_number(N_size)
    double precision::c_mass(N_size,N_aerosol)
    double precision::frac_bin(N_sizebin,N_aerosol)
    double precision::dm_bin(N_sizebin,N_aerosol)
    integer::iclip_bin(N_sizebin,N_aerosol)
    integer::k
  
    frac_bin=0.d0
    dm_bin=0.d0
    iclip_bin=0
    
    do s=1, nesp_eq
      jesp=eq_species(s)
      iclip=0
      do j=1,end_bin!judgment
	  frac(j,jesp)= AAi(j,jesp)*c_number(j)/ce_kernal_coef_tot(jesp)
	  temp_mass=c_mass(j,jesp)+dq(jesp)*frac(j,jesp)
	  if(temp_mass.lt.0.d0) iclip=1!case of over evaporation
      enddo
      if(iclip.eq.1) then !over evaporate
	totaer=0.d0
	do j=1,end_bin
	  totaer=totaer+c_mass(j,jesp)
	enddo
	if(totaer.gt.0.d0) then
	  do j=1,end_bin
	    frac(j,jesp)=c_mass(j,jesp)/totaer
	    c_mass(j,jesp)=c_mass(j,jesp)+dq(jesp)*frac(j,jesp)
	    if(dq(jesp)*frac(j,jesp).ne.0.d0) then
	      k=concentration_index(j,1)
	      frac_bin(k,jesp)=frac_bin(k,jesp)+frac(j,jesp)
	      dm_bin(k,jesp)=dm_bin(k,jesp)+dq(jesp)*frac(j,jesp)
	      iclip_bin(k,jesp)=iclip
	    endif
	  enddo
	endif
      else!normal case
	do j=1,end_bin
	  frac(j,jesp)= AAi(j,jesp)*c_number(j)/ce_kernal_coef_tot(jesp)
	  c_mass(j,jesp)=c_mass(j,jesp)+dq(jesp)*frac(j,jesp)
	  if(dq(jesp)*frac(j,jesp).ne.0.d0) then
	    k=concentration_index(j,1)
	    frac_bin(k,jesp)=frac_bin(k,jesp)+frac(j,jesp)
	    dm_bin(k,jesp)=dm_bin(k,jesp)+dq(jesp)*frac(j,jesp)
	    iclip_bin(k,jesp)=iclip
	  endif
	enddo
      endif
    enddo

  end subroutine bulkequi_redistribution
  
end Module Bulkequibrium
  
  